{+--------------------------------------------------------------------------+
   Component:   Example of Late Customisation using a Page Producer
   Author:      Adrian Rye
 +--------------------------------------------------------------------------+}
unit uLateCustom;

interface

uses
  // Delphi
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DB, Grids, DBGrids, DBTables, ComCtrls, StdCtrls, HTTPApp,
  HTTPProd, Menus,
  ImgList;

type
  TfmMain = class(TForm)
    pcMain: TPageControl;
    tsRawData: TTabSheet;
    tsProperties: TTabSheet;
    gridOrders: TDBGrid;
    lbProperties: TListBox;
    memoProperties: TMemo;
    tsObjects: TTabSheet;
    lvOrders: TListView;
    ilOrdersLarge: TImageList;
    ilOrdersSmall: TImageList;
    pmOrders: TPopupMenu;
    pmiLargeIcons: TMenuItem;
    pmiSmallIcons: TMenuItem;
    pmiList: TMenuItem;
    pmiDetails: TMenuItem;
    N1: TMenuItem;
    pmiUpdateList: TMenuItem;
    ppProperties: TPageProducer;
    btnSetHintProperties: TButton;
    N2: TMenuItem;
    pmiLateCustomisation: TMenuItem;
    procedure lbPropertiesDblClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnSetHintPropertiesClick(Sender: TObject);
    procedure pmiLargeIconsClick(Sender: TObject);
    procedure pmiSmallIconsClick(Sender: TObject);
    procedure pmiListClick(Sender: TObject);
    procedure pmiDetailsClick(Sender: TObject);
    procedure pmiUpdateListClick(Sender: TObject);
    procedure lvOrdersInfoTip(Sender: TObject; Item: TListItem;
      var InfoTip: String);
    procedure ppPropertiesHTMLTag(Sender: TObject; Tag: TTag;
      const TagString: String; TagParams: TStrings;
      var ReplaceText: String);
    procedure pmiLateCustomisationClick(Sender: TObject);
  private
    FUseLateCustomisation: Boolean;
    procedure DoShowHint(var HintStr: string; var CanShow: Boolean;
      var HintInfo: THintInfo);
    procedure SetUseLateCustomisation(const Value: Boolean);
  public
    property UseLateCustomisation: Boolean read FUseLateCustomisation write SetUseLateCustomisation;
  end;

var
  fmMain: TfmMain;

implementation

{$R *.dfm}

uses
  // Delphi
  TypInfo,
  // Late Custom
  uLateCustom_DM, uLateCustom_Order;

var
  TargetObject : TlcOrder;

procedure TfmMain.FormCreate(Sender: TObject);
  begin
    Application.OnShowHint := DoShowHint;
  end;

procedure TfmMain.FormDestroy(Sender: TObject);
  begin
    Application.OnHint := nil;
  end;

procedure TfmMain.lbPropertiesDblClick(Sender: TObject);
  begin
    // adds a formatted property to layout
    if lbProperties.ItemIndex > -1 then begin
      memoProperties.SelText := Format('<#%s>',[lbProperties.Items[lbProperties.ItemIndex]]);
    end;
  end;

procedure TfmMain.DoShowHint(var HintStr: string; var CanShow: Boolean;
  var HintInfo: THintInfo);
  begin
    if HintInfo.HintControl = gridOrders then begin
      with HintInfo do begin
        HintStr     := DMLateCustom.NewHint;
        HintColor   := clMoneyGreen;
        HideTimeout := 5000;
        Inc(HintPos.Y,-25);
        Inc(HintPos.X, 15);
      end;
    end;
  end;


procedure TfmMain.btnSetHintPropertiesClick(Sender: TObject);
  begin
    // update display layout
    DMLateCustom.NewHintProperties    := fmMain.memoProperties.Lines.Text;
  end;

procedure TfmMain.pmiLargeIconsClick(Sender: TObject);
  begin
    lvOrders.ViewStyle := vsIcon;
  end;

procedure TfmMain.pmiSmallIconsClick(Sender: TObject);
  begin
    lvOrders.ViewStyle := vsSmallIcon;
  end;

procedure TfmMain.pmiListClick(Sender: TObject);
  begin
    lvOrders.ViewStyle := vsList;
  end;

procedure TfmMain.pmiDetailsClick(Sender: TObject);
  begin
    lvOrders.ViewStyle := vsReport;
  end;

procedure TfmMain.pmiUpdateListClick(Sender: TObject);
  var
    i : Integer;
  begin
    // initialise Objects
    OrderList.FillList;
    // update Object list view
    lvOrders.Clear;
    for i := 0 to pred(OrderList.Count) do begin
      with lvOrders.Items.Add do begin
        Caption := FloatToStr(OrderList.Order[i].OrderNo);
        SubItems.Add(FloatToStr(OrderList.Order[i].CustNo));
        SubItems.Add(DateToStr(OrderList.Order[i].SaleDate));
        SubItems.Add(DateToStr(OrderList.Order[i].ShipDate));
        SubItems.Add(IntToStr(OrderList.Order[i].EmpNo));
        Data := OrderList.Order[i];
      end;
    end;
  end;

procedure TfmMain.lvOrdersInfoTip(Sender: TObject; Item: TListItem;
  var InfoTip: String);
  begin
    TargetObject := TlcOrder(Item.Data);
    if UseLateCustomisation then begin
      // Late Customisation
      InfoTip := ppProperties.ContentFromString(DMLateCustom.NewHintProperties);
    end else begin
      // Early Customisation
      InfoTip := 'Order ' + FloatToStr(TargetObject.OrderNo) + #13 +
                 'Sold '  + DateToStr(TargetObject.SaleDate);
    end;
  end;

procedure TfmMain.ppPropertiesHTMLTag(Sender: TObject; Tag: TTag;
  const TagString: String; TagParams: TStrings; var ReplaceText: String);
  var
    aDate : TDate;
  begin
    // object
    if Assigned(TargetObject) then begin
      if IsPublishedProp(TargetObject, TagString) then begin
        case PropType(TargetObject, TagString) of
          tkUnknown,tkChar,tkSet,tkClass,tkMethod,tkWChar,
          tkVariant,tkArray,tkRecord,tkInterface,tkInt64,
          tkDynArray :; // do nothing

          tkFloat :
            begin
              if GetPropInfo(TargetObject, TagString)^.PropType^ = TypeInfo(TDateTime) then begin
                aDate := GetPropValue(TargetObject, TagString);
                ReplaceText := DateToStr(aDate);
              end else begin
                ReplaceText := GetPropValue(TargetObject, TagString);
              end;
            end;

          tkInteger,
          tkEnumeration,
          tkString,
          tkLString,
          tkWString :
            begin
              ReplaceText := GetPropValue(TargetObject, TagString);
            end;
        end;
      end else begin
        ReplaceText := '????';
      end;
    end;
  end;

procedure TfmMain.SetUseLateCustomisation(const Value: Boolean);
  begin
    FUseLateCustomisation             := Value;
    DMLateCustom.UseLateCustomisation := Value;
  end;

procedure TfmMain.pmiLateCustomisationClick(Sender: TObject);
  begin
    pmiLateCustomisation.Checked := not pmiLateCustomisation.Checked;
    UseLateCustomisation := pmiLateCustomisation.Checked;
  end;

end.
